@@ -29,6 +29,7 @@
 
 #include <osg/Geode>
 #include <osg/MatrixTransform>
+#include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 #include <osgDB/Registry>
 #include <osg/Switch>
@@ -40,11 +41,10 @@
 #include <simgear/props/props_io.hxx>
 #include <simgear/props/condition.hxx>
 #include <simgear/scene/util/SGNodeMasks.hxx>
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
 
 #include "modellib.hxx"
-#include "SGPagedLOD.hxx"
 #include "SGReaderWriterXML.hxx"
-#include "SGReaderWriterXMLOptions.hxx"
 
 #include "animation.hxx"
 #include "particles.hxx"
@@ -208,8 +208,8 @@ sgLoad3DModel_internal(const SGPath& path,
       return NULL;
     }
     
-    const SGReaderWriterXMLOptions* xmlOptions;
-    xmlOptions = dynamic_cast<const SGReaderWriterXMLOptions*>(options_);
+    const SGReaderWriterOptions* xmlOptions;
+    xmlOptions = dynamic_cast<const SGReaderWriterOptions*>(options_);
 
     SGSharedPtr<SGPropertyNode> prop_root;
     osg::Node *(*load_panel)(SGPropertyNode *)=0;
@@ -219,7 +219,7 @@ sgLoad3DModel_internal(const SGPath& path,
     SGPath modelDir(modelpath.dir());
     
     if (xmlOptions) {
-        prop_root = xmlOptions->getPropRoot();
+        prop_root = xmlOptions->getPropertyNode();
         load_panel = xmlOptions->getLoadPanel();
         data = xmlOptions->getModelData();
     }
@@ -269,9 +269,9 @@ sgLoad3DModel_internal(const SGPath& path,
         // model without wrapper
     }
 
-    osg::ref_ptr<SGReaderWriterXMLOptions> options
-    = new SGReaderWriterXMLOptions(*options_);
-    options->setPropRoot(prop_root);
+    osg::ref_ptr<SGReaderWriterOptions> options
+    = new SGReaderWriterOptions(*options_);
+    options->setPropertyNode(prop_root);
     options->setLoadPanel(load_panel);
     
     // Assume that textures are in
@@ -281,9 +281,8 @@ sgLoad3DModel_internal(const SGPath& path,
             texturepath = texturepath.dir();
 
         options->setDatabasePath(texturepath.str());
-        osgDB::ReaderWriter::ReadResult modelResult
-            = osgDB::Registry::instance()->readNode(modelpath.str(),
-                                                    options.get());
+        osgDB::ReaderWriter::ReadResult modelResult;
+        modelResult = osgDB::readNodeFile(modelpath.str(), options.get());
         if (!modelResult.validNode())
             throw sg_io_exception("Failed to load 3D model:" + modelResult.message(),
                                   modelpath.str());
@@ -354,9 +353,9 @@ sgLoad3DModel_internal(const SGPath& path,
           continue;
         }
 
-        osg::ref_ptr<SGReaderWriterXMLOptions> options;
-        options = new SGReaderWriterXMLOptions(*options_);
-        options->setPropRoot(prop_root);
+        osg::ref_ptr<SGReaderWriterOptions> options;
+        options = new SGReaderWriterOptions(*options_);
+        options->setPropertyNode(prop_root);
         options->setLoadPanel(load_panel);
         
         try {
